package id

import (
	"strings"

	jsoniter "github.com/json-iterator/go"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/bsonrw"
	"go.mongodb.org/mongo-driver/bson/bsontype"
)

const Separator = '/'

type Descriptor interface {
	New() Descriptor
	String() string
	Type() string
	Map() map[string]any
	Validate() error
	FromMap(map[string]any) error
	FromParts([]string) error
}

type ObjectId struct {
	Descriptor
}

func NewObjectId(in any) (*ObjectId, error) {
	switch v := in.(type) {
	case Descriptor:
		return &ObjectId{Descriptor: v}, nil
	case string:
		return FromString(v)
	case map[string]any:
		return FromMap(v)
	}
	return FromObject(in)
}

func MustObjectId(in any) *ObjectId {
	oid, err := NewObjectId(in)
	if err != nil {
		panic(err)
	}
	return oid
}

func (oid *ObjectId) MarshalJSON() ([]byte, error) {
	return jsoniter.Marshal(oid.String())
}

func (oid *ObjectId) UnmarshalJSON(b []byte) error {
	var s string
	var err error
	if err = jsoniter.Unmarshal(b, &s); err != nil {
		return err
	}
	t, err := FromString(s)
	if err != nil {
		return err
	}
	*oid = *t
	return nil
}

func (oid *ObjectId) MarshalBSONValue() (bsontype.Type, []byte, error) {
	return bson.MarshalValue(oid.String())
}

func (oid *ObjectId) UnmarshalBSONValue(btype bsontype.Type, data []byte) error {
	if btype != bson.TypeString {
		return ErrInvalidType
	}
	dec, err := bson.NewDecoder(bsonrw.NewBSONValueReader(btype, data))
	if err != nil {
		return err
	}
	var str string
	if err = dec.Decode(&str); err != nil {
		return err
	}
	t, err := FromString(str)
	if err != nil {
		return err
	}
	*oid = *t
	return nil
}

func Join(parts ...string) string {
	s := strings.Join(parts, string(Separator))
	if s[0] != Separator {
		s = string(Separator) + s
	}
	return s
}

func Split(id string) []string {
	if id[0] != Separator {
		return nil
	}
	return strings.Split(id[1:], string(Separator))
}
